/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef JAC_AP_VISUALIZATION_VISUALIZATION_PROXY_H
#define JAC_AP_VISUALIZATION_VISUALIZATION_PROXY_H

#include "ara/com/internal/proxy/proxy_adapter.h"
#include "ara/com/internal/proxy/event_adapter.h"
#include "ara/com/internal/proxy/field_adapter.h"
#include "ara/com/internal/proxy/method_adapter.h"
#include "ara/com/crc_verification.h"
#include "jac/ap/visualization/visualization_common.h"
#include <string>

namespace jac {
namespace ap {
namespace visualization {
namespace proxy {
namespace events {
    using CameraH265ImagePDataViz = ara::com::internal::proxy::event::EventAdapter<::CameraH265Image>;
    using LidarHesaiRightPPC2Viz = ara::com::internal::proxy::event::EventAdapter<::perception::LiDAR2LiDARPerception>;
    using LidarInnoPPC2Viz = ara::com::internal::proxy::event::EventAdapter<::perception::LiDAR2LiDARPerception>;
    using LidarInno2PPC2Viz = ara::com::internal::proxy::event::EventAdapter<::perception::LiDAR2LiDARPerception>;
    using LidarHesaiRight2PPC2Viz = ara::com::internal::proxy::event::EventAdapter<::perception::LiDAR2LiDARPerception>;
    using LidarHesaiLeftPPC2Viz = ara::com::internal::proxy::event::EventAdapter<::perception::LiDAR2LiDARPerception>;
    using LidarHesaiLeft2PPC2Viz = ara::com::internal::proxy::event::EventAdapter<::perception::LiDAR2LiDARPerception>;
    static constexpr ara::com::internal::EntityId VisualizationCameraH265ImagePDataVizId = 35045U; //CameraH265ImagePDataViz_event_hash
    static constexpr ara::com::internal::EntityId VisualizationLidarHesaiRightPPC2VizId = 39353U; //LidarHesaiRightPPC2Viz_event_hash
    static constexpr ara::com::internal::EntityId VisualizationLidarInnoPPC2VizId = 24796U; //LidarInnoPPC2Viz_event_hash
    static constexpr ara::com::internal::EntityId VisualizationLidarInno2PPC2VizId = 43181U; //LidarInno2PPC2Viz_event_hash
    static constexpr ara::com::internal::EntityId VisualizationLidarHesaiRight2PPC2VizId = 25020U; //LidarHesaiRight2PPC2Viz_event_hash
    static constexpr ara::com::internal::EntityId VisualizationLidarHesaiLeftPPC2VizId = 36987U; //LidarHesaiLeftPPC2Viz_event_hash
    static constexpr ara::com::internal::EntityId VisualizationLidarHesaiLeft2PPC2VizId = 38317U; //LidarHesaiLeft2PPC2Viz_event_hash
}

namespace fields {
}

namespace methods {

} // namespace methods

class VisualizationProxy {
private:
    std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> proxyAdapter;
public:
    using HandleType = ara::com::HandleType;
    class ConstructionToken {
    public:
        explicit ConstructionToken(const std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> & proxy)
            : proxyAdapter(proxy),
              CameraH265ImagePDataViz(proxyAdapter->GetProxy(), events::VisualizationCameraH265ImagePDataVizId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
              LidarHesaiRightPPC2Viz(proxyAdapter->GetProxy(), events::VisualizationLidarHesaiRightPPC2VizId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
              LidarInnoPPC2Viz(proxyAdapter->GetProxy(), events::VisualizationLidarInnoPPC2VizId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
              LidarInno2PPC2Viz(proxyAdapter->GetProxy(), events::VisualizationLidarInno2PPC2VizId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
              LidarHesaiRight2PPC2Viz(proxyAdapter->GetProxy(), events::VisualizationLidarHesaiRight2PPC2VizId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
              LidarHesaiLeftPPC2Viz(proxyAdapter->GetProxy(), events::VisualizationLidarHesaiLeftPPC2VizId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
              LidarHesaiLeft2PPC2Viz(proxyAdapter->GetProxy(), events::VisualizationLidarHesaiLeft2PPC2VizId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()){
        }

        ConstructionToken(ConstructionToken&& other) = default;
        ConstructionToken& operator=(ConstructionToken && other) = default;

        ConstructionToken(const ConstructionToken&) = delete;
        ConstructionToken& operator = (const ConstructionToken&) = delete;
        
        virtual ~ConstructionToken() = default;

        /* Do not use such internal interfaces!!! */
        ara::core::Result<void> InitializeAll()
        {
            ara::core::Result<void> initResult;
            return initResult;
        }
        std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> GetProxyAdapter()
        {
            return proxyAdapter;
        }
        events::CameraH265ImagePDataViz GetCameraH265ImagePDataViz() noexcept { return std::move(CameraH265ImagePDataViz); }
        events::LidarHesaiRightPPC2Viz GetLidarHesaiRightPPC2Viz() noexcept { return std::move(LidarHesaiRightPPC2Viz); }
        events::LidarInnoPPC2Viz GetLidarInnoPPC2Viz() noexcept { return std::move(LidarInnoPPC2Viz); }
        events::LidarInno2PPC2Viz GetLidarInno2PPC2Viz() noexcept { return std::move(LidarInno2PPC2Viz); }
        events::LidarHesaiRight2PPC2Viz GetLidarHesaiRight2PPC2Viz() noexcept { return std::move(LidarHesaiRight2PPC2Viz); }
        events::LidarHesaiLeftPPC2Viz GetLidarHesaiLeftPPC2Viz() noexcept { return std::move(LidarHesaiLeftPPC2Viz); }
        events::LidarHesaiLeft2PPC2Viz GetLidarHesaiLeft2PPC2Viz() noexcept { return std::move(LidarHesaiLeft2PPC2Viz); }
    private:
        std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> proxyAdapter;
        events::CameraH265ImagePDataViz CameraH265ImagePDataViz;
        events::LidarHesaiRightPPC2Viz LidarHesaiRightPPC2Viz;
        events::LidarInnoPPC2Viz LidarInnoPPC2Viz;
        events::LidarInno2PPC2Viz LidarInno2PPC2Viz;
        events::LidarHesaiRight2PPC2Viz LidarHesaiRight2PPC2Viz;
        events::LidarHesaiLeftPPC2Viz LidarHesaiLeftPPC2Viz;
        events::LidarHesaiLeft2PPC2Viz LidarHesaiLeft2PPC2Viz;
    };

    virtual ~VisualizationProxy()
    {
        CameraH265ImagePDataViz.UnsetReceiveHandler();
        CameraH265ImagePDataViz.Unsubscribe();
        LidarHesaiRightPPC2Viz.UnsetReceiveHandler();
        LidarHesaiRightPPC2Viz.Unsubscribe();
        LidarInnoPPC2Viz.UnsetReceiveHandler();
        LidarInnoPPC2Viz.Unsubscribe();
        LidarInno2PPC2Viz.UnsetReceiveHandler();
        LidarInno2PPC2Viz.Unsubscribe();
        LidarHesaiRight2PPC2Viz.UnsetReceiveHandler();
        LidarHesaiRight2PPC2Viz.Unsubscribe();
        LidarHesaiLeftPPC2Viz.UnsetReceiveHandler();
        LidarHesaiLeftPPC2Viz.Unsubscribe();
        LidarHesaiLeft2PPC2Viz.UnsetReceiveHandler();
        LidarHesaiLeft2PPC2Viz.Unsubscribe();

    }

    explicit VisualizationProxy(const HandleType &handle)
        : proxyAdapter(std::make_shared<ara::com::internal::proxy::ProxyAdapter>(::jac::ap::visualization::Visualization::ServiceIdentifier, handle)),
          CameraH265ImagePDataViz(proxyAdapter->GetProxy(), events::VisualizationCameraH265ImagePDataVizId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
          LidarHesaiRightPPC2Viz(proxyAdapter->GetProxy(), events::VisualizationLidarHesaiRightPPC2VizId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
          LidarInnoPPC2Viz(proxyAdapter->GetProxy(), events::VisualizationLidarInnoPPC2VizId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
          LidarInno2PPC2Viz(proxyAdapter->GetProxy(), events::VisualizationLidarInno2PPC2VizId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
          LidarHesaiRight2PPC2Viz(proxyAdapter->GetProxy(), events::VisualizationLidarHesaiRight2PPC2VizId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
          LidarHesaiLeftPPC2Viz(proxyAdapter->GetProxy(), events::VisualizationLidarHesaiLeftPPC2VizId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
          LidarHesaiLeft2PPC2Viz(proxyAdapter->GetProxy(), events::VisualizationLidarHesaiLeft2PPC2VizId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()){
        }

    void ThrowError(const ara::core::Result<void>& result) const
    {
        if (!(result.HasValue())) {
#ifndef NOT_SUPPORT_EXCEPTIONS
            ara::core::ErrorCode errorcode(result.Error());
            throw ara::com::ComException(std::move(errorcode));
#else
            std::cerr << "Error: Not support exception, create proxy failed!"<< std::endl;
#endif
        }
    }

    VisualizationProxy(const VisualizationProxy&) = delete;
    VisualizationProxy& operator=(const VisualizationProxy&) = delete;

    VisualizationProxy(VisualizationProxy&&) = default;
    VisualizationProxy& operator=(VisualizationProxy&&) = default;
    VisualizationProxy(ConstructionToken&& token) noexcept
        : proxyAdapter(token.GetProxyAdapter()),
          CameraH265ImagePDataViz(token.GetCameraH265ImagePDataViz()),
          LidarHesaiRightPPC2Viz(token.GetLidarHesaiRightPPC2Viz()),
          LidarInnoPPC2Viz(token.GetLidarInnoPPC2Viz()),
          LidarInno2PPC2Viz(token.GetLidarInno2PPC2Viz()),
          LidarHesaiRight2PPC2Viz(token.GetLidarHesaiRight2PPC2Viz()),
          LidarHesaiLeftPPC2Viz(token.GetLidarHesaiLeftPPC2Viz()),
          LidarHesaiLeft2PPC2Viz(token.GetLidarHesaiLeft2PPC2Viz()){
        static_cast<void>(std::move(token));
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        const ara::com::HandleType &handle)
    {
        std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> preProxyAdapter =
            std::make_shared<ara::com::internal::proxy::ProxyAdapter>(
               ::jac::ap::visualization::Visualization::ServiceIdentifier, handle);
        ConstructionToken token{std::move(preProxyAdapter)};
        ara::core::Result<void> initResult {token.InitializeAll()};
        if (initResult.HasValue()) {
            return ara::core::Result<ConstructionToken>(std::move(token));
        } else {
            const ara::core::ErrorCode errorcode{initResult.Error()};
            ara::core::Result<VisualizationProxy::ConstructionToken> preResult{errorcode};
            return preResult;
        }
    }

    static ara::com::FindServiceHandle StartFindService(
        const ara::com::FindServiceHandler<ara::com::internal::proxy::ProxyAdapter::HandleType>& handler,
        const ara::com::InstanceIdentifier instance)
    {
        return ara::com::internal::proxy::ProxyAdapter::StartFindService(handler, ::jac::ap::visualization::Visualization::ServiceIdentifier, instance);
    }

    static ara::com::FindServiceHandle StartFindService(
        const ara::com::FindServiceHandler<ara::com::internal::proxy::ProxyAdapter::HandleType> handler,
        const ara::core::InstanceSpecifier specifier)
    {
        return ara::com::internal::proxy::ProxyAdapter::StartFindService(handler, ::jac::ap::visualization::Visualization::ServiceIdentifier, specifier);
    }

    static ara::com::ServiceHandleContainer<ara::com::internal::proxy::ProxyAdapter::HandleType> FindService(
        const ara::com::InstanceIdentifier instance)
    {
        return ara::com::internal::proxy::ProxyAdapter::FindService(::jac::ap::visualization::Visualization::ServiceIdentifier, instance);
    }

    static ara::com::ServiceHandleContainer<ara::com::internal::proxy::ProxyAdapter::HandleType> FindService(
        const ara::core::InstanceSpecifier specifier)
    {
        return ara::com::internal::proxy::ProxyAdapter::FindService(::jac::ap::visualization::Visualization::ServiceIdentifier, specifier);
    }

    static void StopFindService(const ara::com::FindServiceHandle& handle)
    {
        ara::com::internal::proxy::ProxyAdapter::StopFindService(handle);
    }

    HandleType GetHandle() const
    {
        return proxyAdapter->GetHandle();
    }
    bool SetEventThreadNumber(const std::uint16_t number, const std::uint16_t queueSize)
    {
        return proxyAdapter->SetEventThreadNumber(number, queueSize);
    }
    events::CameraH265ImagePDataViz CameraH265ImagePDataViz;
    events::LidarHesaiRightPPC2Viz LidarHesaiRightPPC2Viz;
    events::LidarInnoPPC2Viz LidarInnoPPC2Viz;
    events::LidarInno2PPC2Viz LidarInno2PPC2Viz;
    events::LidarHesaiRight2PPC2Viz LidarHesaiRight2PPC2Viz;
    events::LidarHesaiLeftPPC2Viz LidarHesaiLeftPPC2Viz;
    events::LidarHesaiLeft2PPC2Viz LidarHesaiLeft2PPC2Viz;
};
} // namespace proxy
} // namespace visualization
} // namespace ap
} // namespace jac

#endif // JAC_AP_VISUALIZATION_VISUALIZATION_PROXY_H
